import json
import subprocess

import pytest


@pytest.mark.integration
class TestIntegrationDeleteFinding(object):
    def test_delete_and_push_findings(self):
        p = subprocess.Popen(
            ["reptor", "deletefindings", "--no-dry-run"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        p.communicate()

        for i in range(3):
            finding = {
                "status": "in-progress",
                "data": {
                    "title": f"Finding Title {i}",
                },
            }
            p = subprocess.Popen(
                ["reptor", "finding"],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                stdin=subprocess.PIPE,
            )
            out, err = p.communicate(input=json.dumps(finding).encode())
            assert p.returncode == 0

    @pytest.mark.parametrize(
        "title_contains,exclude_title_contains,expected",
        [
            ("Finding", "", ["Finding Title 0", "Finding Title 1", "Finding Title 2"]),
            ("Finding", "1", ["Finding Title 0", "Finding Title 2"]),
            ("", "2", ["Finding Title 0", "Finding Title 1"]),
            ("", "", ["Finding Title 0", "Finding Title 1", "Finding Title 2"]),
            ("1", "", ["Finding Title 1"]),
            ("1", "1", []),
            ("", "Finding", []),
        ],
    )
    def test_delete_dry_run(self, title_contains, exclude_title_contains, expected):
        p = subprocess.Popen(
            [
                "reptor",
                "deletefindings",
                "--title-contains",
                title_contains,
                "--exclude-title-contains",
                exclude_title_contains,
            ],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        _, output = p.communicate()
        assert p.returncode == 0
        for title in expected:
            assert title in output.decode()

    def test_delete(self):
        p = subprocess.Popen(
            [
                "reptor",
                "deletefindings",
                "--no-dry-run",
            ],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        _, output = p.communicate()
        assert p.returncode == 0
        for i in range(3):
            assert f"Finding Title {i}" in output.decode()



import subprocess
import time

import pytest

from reptor.plugins.core.Conf.tests.conftest import (
    get_note,
    private_uploads_id,
    private_notes_api
)


@pytest.mark.integration
class TestIntegrationNote(object):
    def test_notes_upload_without_notetitle(self):
        # Upload content to "Uploads"
        note_content = str(time.time()).encode()
        p = subprocess.Popen(
            ["reptor", "note", "--private-note"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            stdin=subprocess.PIPE,
        )
        out, err = p.communicate(note_content)
        assert p.returncode == 0

        # Check if content was added to "Uploads"
        note = get_note("Uploads", None, private=True)
        assert note is not None
        assert note_content.decode() in note["text"]

    def test_notes_upload_with_notetitle(self, private_uploads_id):
        note_content = str(time.time()).encode()
        # Upload to note with notetitle
        p = subprocess.Popen(
            ["reptor", "note", "--private-note", "--notetitle", note_content.decode()],
            stdin=subprocess.PIPE,
        )
        p.communicate(note_content)
        assert p.returncode == 0

        # Check if content was added to note with notetitle
        note = get_note(note_content.decode(), private_uploads_id, private=True)
        assert note is not None
        assert note_content.decode() in note["text"]



import requests

from reptor.lib.importers.BaseImporter import BaseImporter
from reptor.models.UserConfig import UserConfig


class DefectDojo(BaseImporter):
    """
    Imports findings from DefectDojo

    Connects to the GraqhQL API of a DefectDojo instance and imports its
    finding templates to SysReptor via API.
    """

    meta = {
        "author": "Julian Fonticoba",
        "name": "DefectDojo",
        "version": "1.0",
        "license": "MIT",
        "summary": "Imports DefectDojo finding templates",
    }

    # TODO
    mapping = {
        "title": "title",
        "cvssv3_score": "cvss",
        "description": "summary",
        "severity": "severity",
        "mitigation": "recommendation",
        "references": "references",
    }
    defectdojo_url: str
    apikey: str

    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)
        if self.conf:
            return

        self.defectdojo_url = kwargs.get("url", "")
        if not self.defectdojo_url:
            try:
                self.defectdojo_url = self.url
            except AttributeError:
                raise ValueError("DefectDojo URL is required.")
        if not hasattr(self, "apikey"):
            raise ValueError("DefectDojo API Key is required. Add to your user config.")

    @property
    def user_config(self):
        return [
            UserConfig(
                name="url",
                friendly_name="DefectDojo URL",
            ),
            UserConfig(
                name="apikey",
                friendly_name="DefectDojo API key v2",
                redact_current_value=True,
            ),
        ]

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath=plugin_filepath)
        action_group = parser.add_argument_group()
        action_group.add_argument(
            "--url",
            metavar="URL",
            action="store",
            const="",
            nargs="?",
            help="DefectDojo API",
        )

    def strip_references(self, text):
        if not text:
            return []
        return text.splitlines()

    def _get_defectdojo_findings(self, url=None):
        # Include the Authorization Token that it may be used
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Token {self.apikey}",
        }
        return requests.get(
            url
            or f"{self.defectdojo_url}/api/v2/findings/?active=true&false_p=false&is_mitigated=false",
            headers=headers,
            timeout=60,
            verify=not self.reptor.get_config().get("insecure", False),
        ).json()

    def next_findings_batch(self):
        self.debug("Running batch findings")

        next = None
        while True:
            response = self._get_defectdojo_findings(next)
            findings = response.get("results", [])
            for finding_data in findings:
                finding_data["references"] = self.strip_references(
                    finding_data.get("references")
                )
                finding_data["severity"] = (finding_data.get("severity") or "").lower()
                yield {
                    "language": "en-US",
                    "is_main": True,
                    "status": "in-progress",
                    "data": finding_data,
                }
            if not (next := response.get("next")):
                break


loader = DefectDojo



import os
from pathlib import Path

import pytest

from reptor.lib.plugins.TestCaseToolPlugin import TestCaseToolPlugin
from reptor.models.Note import NoteTemplate

from ..OpenVAS import OpenVAS


class TestOpenVAS(TestCaseToolPlugin):
    templates_path = os.path.normpath(Path(os.path.dirname(__file__)) / "../templates")

    @pytest.fixture(autouse=True)
    def setUp(self) -> None:
        OpenVAS.setup_class(
            Path(os.path.dirname(self.templates_path)), skip_user_plugins=True
        )
        self.openvas = OpenVAS()

    def _load_xml_data(self, filename):
        self.openvas.input_format = "xml"
        filepath = os.path.join(os.path.dirname(__file__), f"./data/{filename}.xml")
        with open(filepath, "r") as f:
            self.openvas.raw_input = f.read()

    def test_parse_multi_input(self):
        self._load_xml_data("openvas")
        self.openvas.raw_input = [self.openvas.raw_input, self.openvas.raw_input]
        self.openvas.parse()
        assert len(self.openvas.parsed_input) == 72

    def test_parse_with_qod_filter(self):
        self._load_xml_data("openvas")
        self.openvas.min_qod = 50
        self.openvas.parse()
        p = self.openvas.parsed_input
        assert isinstance(p, list)
        assert len(p) == 5

    def test_parse_with_filter(self):
        self._load_xml_data("openvas")
        self.openvas.severity_filter = {"critical"}
        self.openvas.parse()
        p = self.openvas.parsed_input
        assert isinstance(p, list)
        assert len(p) == 11

    def test_preprocess_for_template(self):
        self._load_xml_data("openvas")
        self.openvas.parse()
        p = self.openvas.preprocess_for_template()
        assert len(p) == 36
        assert p[0]["affected_components"] == ["10.20.0.125"]
        assert p[0]["risk_factor"] == "critical"
        assert p[0]["oid"] == "1.3.6.1.4.1.25623.1.0.103674"
        assert p[0]["oid"] in p[0]["finding_templates"]
        assert p[0]["cvss_vector"] == "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H"
        assert len(set([f["name"] for f in p])) == 36
        assert all(isinstance(f.get("affected_components"), list) for f in p)

    def test_aggregate_by_plugin(self):
        self._load_xml_data("openvas")
        self.openvas.parse()
        a = self.openvas.aggregate_by_plugin()
        assert len(a) == 36
        assert all([len(v) == 1 for v in a])

    def test_create_notes(self):
        self._load_xml_data("openvas")
        self.openvas.parse()
        note = self.openvas.create_notes()
        assert isinstance(note, NoteTemplate)
        assert note.icon_emoji == "🦖"
        assert note.title == "OpenVAS"
        assert note.children[0].title == "10.20.0.125"
        assert len(note.children[0].children) == 36

        # Test "get_reports_response"
        self._load_xml_data("openvas")
        assert self.openvas.raw_input is not None
        self.openvas.raw_input = f"""<get_reports_response status="200" status_text="OK">{self.openvas.raw_input}</get_reports_response>"""
        self.openvas.parse()
        note = self.openvas.create_notes()
        assert isinstance(note, NoteTemplate)
        assert note.icon_emoji == "🦖"
        assert note.title == "OpenVAS"
        assert note.children[0].title == "10.20.0.125"
        assert len(note.children[0].children) == 36

    def test_aggregate_by_target(self):
        self._load_xml_data("openvas")
        self.openvas.parse()
        a = self.openvas.aggregate_by_target()
        assert len(a) == 1
        assert len(a[0]) == 36

    def test_parse(self):
        self._load_xml_data("openvas")
        self.openvas.parse()
        p = self.openvas.parsed_input
        assert isinstance(p, list)
        assert len(p) == 36

        # Check sorting
        severity = 10
        for finding in p:
            assert finding["severity"] <= severity
            severity = finding["severity"]

        assert p[0]["target"] == "10.20.0.125"
        assert p[0]["risk_factor"] == "critical"
        assert p[0]["host"]["ip"] == "10.20.0.125"
        assert p[0]["nvt"]["oid"] == "1.3.6.1.4.1.25623.1.0.103674"
        assert p[0]["nvt"]["tags"]["cvss_base_vector"] == "AV:N/AC:L/Au:N/C:C/I:C/A:C"
        assert p[0]["nvt"]["tags"]["summary"].startswith("The Operating System (OS)")



import argparse
import json
import os
import tarfile
import tempfile
from pathlib import Path
from shutil import copytree
from typing import Any
from io import FileIO

import tomlkit
import tomlkit.items

from reptor.lib.plugins.Base import Base


def to_toml(data: Any):
    if isinstance(data, dict):
        table = tomlkit.table()
        keys_prepend = [
            "title",
            "cvss",
            "severity",
            "summary",
            "impact",
            "description",
            "recommendation",
        ]
        keys_append = ["report_data", "findings", "project_type", "translations"]

        ordered_keys = list(data.keys())
        for k in keys_prepend + keys_append:
            if k in ordered_keys:
                ordered_keys.remove(k)
        for k in reversed(keys_prepend):
            if k in data:
                ordered_keys.insert(0, k)
        for k in keys_append:
            if k in data:
                ordered_keys.append(k)

        for k in ordered_keys:
            if data[k] is not None:
                table.append(k, to_toml(data[k]))
        return table
    elif isinstance(data, list):
        array = (
            tomlkit.aot()
            if (len(data) > 0 and isinstance(data[0], dict))
            else tomlkit.items.Array(
                value=[], trivia=tomlkit.items.Trivia(), multiline=True
            )
        )
        for v in data:
            if v is not None:
                array.append(to_toml(v))
        return array
    elif isinstance(data, bool):
        return tomlkit.items.Bool(data, trivia=tomlkit.items.Trivia())
    elif isinstance(data, int):
        return tomlkit.items.Integer(data, trivia=tomlkit.items.Trivia(), raw=str(data))
    elif isinstance(data, float):
        return tomlkit.items.Float(data, trivia=tomlkit.items.Trivia(), raw=str(data))
    elif isinstance(data, str):
        if "\n" in data:
            if data[0] != "\n":
                data = "\n" + data
            if data[-1] != "\n":
                data += "\n"
            return tomlkit.string(data, multiline=True)
        else:
            return tomlkit.string(data)
    # elif data is None:
    #    # TOML does not support null values
    #    return None
    else:
        raise Exception(f"Unhandled type: {type(data)}")


class UnpackArchive(Base):
    meta = {
        "name": "UnpackArchive",
        "summary": "Unpack .tar.gz exported archives",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.files: list[FileIO] = kwargs.get("files", [])
        self.output = kwargs.get("output") or "./unpacked_archive"
        self.format = kwargs.get("format") or "toml"

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath)

        parser.add_argument("files", nargs="+", type=argparse.FileType("rb"))
        parser.add_argument("-o", "--output")
        parser.add_argument("-f", "--format", choices=["json", "toml"], default="toml")

    def run(self):
        with tempfile.TemporaryDirectory() as tempdir:
            for file in self.files:
                with tarfile.open(fileobj=file, mode="r:gz") as tar:
                    tar.extractall(tempdir)
                for path_json in Path(tempdir).glob("*.json"):
                    if not path_json.is_file():
                        continue
                    data_dict = json.loads(path_json.read_text())
                    if self.format == "json":
                        data_output = json.dumps(data_dict, indent=2)
                    elif self.format == "toml":
                        data_output = tomlkit.dumps(to_toml(data_dict))
                    path_output = path_json.with_suffix(f".{self.format}")
                    path_output.write_text(data_output)
                    if path_output != path_json:
                        path_json.unlink()

            self.output = os.path.abspath(self.output)
            copytree(src=tempdir, dst=self.output, dirs_exist_ok=True)
            self.success(f"Unpacked files to {self.output}")


loader = UnpackArchive



import pathlib
import typing


class PluginMeta:
    _overwrites = None

    name: str = ""
    author: str = ""
    version: str = ""
    website: str = ""
    license: str = ""
    tags: list = []
    summary: str = ""
    category: str = ""

    path: typing.Optional[pathlib.Path] = None

    def __init__(self, meta_dictionary: typing.Dict):
        self.author = meta_dictionary.get("author", "")
        self.version = meta_dictionary.get("version", "")
        self.website = meta_dictionary.get("website", "")
        self.license = meta_dictionary.get("license", "")
        self.tags = meta_dictionary.get("tags", [])
        self.summary = meta_dictionary.get("summary", "")

    def set_overwrites_plugin(self, plugin):
        self._overwrites = plugin

    def get_overwritten_plugin(self):
        return self._overwrites



import argparse
import sys

from reptor.lib.plugins.UploadBase import UploadBase
from reptor.api.NotesAPI import NotesAPI


class File(UploadBase):
    """
    Arguments:
    * --file        files to upload; leave empty for stdin
    * --filename|-fn    filename if file provided via stdin

    """

    meta = {
        "name": "File",
        "summary": "Uploads a file",
    }

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath=plugin_filepath)

        parser.add_argument(
            "file",
            nargs="*",
            type=argparse.FileType("r"),
            help="files to upload; leave empty for stdin",
        )
        parser.add_argument(
            "-fn", "--filename", help="filename if file provided via stdin"
        )

    def run(self):
        files = self.reptor.get_config().get("cli").get("file")
        if not files:
            files = [sys.stdin]
        filename = self.reptor.get_config().get("cli").get("filename")
        notetitle = self.reptor.get_config().get("cli").get("notetitle")
        parent_notetitle = None
        icon = None
        if notetitle:
            parent_notetitle = "Uploads"
        else:
            notetitle = "Uploads"
            icon = "📤"
        timestamp = not self.reptor.get_config().get("cli").get("no_timestamp")

        for file in files:
            self.reptor.api.notes.upload_file(
                file=file,
                notetitle=notetitle,
                filename=filename,
                caption=filename,
                parent_notetitle=parent_notetitle,
                timestamp=timestamp,
                icon=icon,
            )


loader = File



import json
import os
import uuid
from unittest.mock import MagicMock, Mock, patch

import pytest
from requests.exceptions import HTTPError

from reptor.lib.reptor import reptor
from reptor.models.FindingTemplate import FindingTemplate
from reptor.models.Project import Project
from reptor.models.ProjectDesign import ProjectDesign

from ..FindingFromTemplate import FindingFromTemplate


class TestFinding:
    finding_template = json.loads(
        open(
            os.path.join(os.path.dirname(__file__), "./data/finding_template.json")
        ).read()
    )

    @pytest.fixture(autouse=True)
    def setup(self):
        self.reptor = reptor
        self.finding_from_template = FindingFromTemplate()

    @pytest.mark.parametrize(
        ["template_id", "raises"],
        [
            ("template_id", True),
            ("1", True),
            (str(uuid.uuid1()), False),
            (str(uuid.uuid3(uuid.NAMESPACE_DNS, "example")), False),
            (str(uuid.uuid4()), False),
            (str(uuid.uuid5(uuid.NAMESPACE_DNS, "example")), False),
        ],
    )
    def test_get_template_by_id(self, template_id, raises):
        self.reptor.api._templates = MagicMock()
        with patch.object(
            self.finding_from_template.reptor.api.templates,
            "get_template",
            side_effect=HTTPError(response=Mock(status_code=404)),
        ) as mock_get_template:
            if raises:
                with pytest.raises(ValueError):
                    self.finding_from_template._get_template_by_id(template_id)
                mock_get_template.assert_not_called()
            else:
                with pytest.raises(KeyError):
                    # If uuid check passses, template doesn't exist
                    self.finding_from_template._get_template_by_id(template_id)
                mock_get_template.assert_called_once()

                # Reset mock and test with existing template
                mock_get_template.reset_mock()
                mock_get_template.side_effect = None
                mock_get_template.return_value = FindingTemplate(self.finding_template)
                assert self.finding_from_template._get_template_by_id(template_id)
                mock_get_template.assert_called_once()

    @pytest.mark.parametrize(
        ["tags", "found"],
        [
            (["tag1"], False),
            (["web"], True),
            (["web", "tag1"], False),
        ],
    )
    def test_get_templates_by_tag(self, tags, found):
        self.reptor.api._templates = MagicMock()
        self.finding_from_template.reptor.api.templates.get_templates_by_tag = (
            MagicMock(return_value=[FindingTemplate(self.finding_template)])
        )
        if not found:
            with pytest.raises(KeyError):
                self.finding_from_template._get_templates_by_tags(tags)
        else:
            assert self.finding_from_template._get_templates_by_tags(tags)

    @pytest.mark.parametrize(
        ["project_language", "expected_language", "expected_index"],
        [
            ("en-US", "en-US", 0),
            ("de-DE", "de-DE", 1),
            ("fr-FR", "en-US", 0),  # Fallback to main translation
            ("", "en-US", 0),
        ],
    )
    def test_get_template_translation(
        self, project_language, expected_language, expected_index
    ):
        self.reptor.api._projects = MagicMock()
        self.reptor.api.projects.project.language = project_language
        assert self.reptor.api.projects.project.language == project_language
        template = FindingTemplate(self.finding_template)
        assert self.finding_from_template._get_template_translation(template) == (
            expected_index,
            expected_language,
        )
        del self.reptor.api.projects.project  # clear cached property



import json
import os
import pathlib
import subprocess

import pytest
import yaml

from reptor.plugins.core.Conf.tests.conftest import (
    notes_api,
    projects_api,
    read_until,
    templates_api,
)


@pytest.mark.integration
class TestIntegrationNessus(object):

    def test_upload_finding_templates(self, templates_api):
        # Delete existing finding templates
        for tag in ["nessus:global", "nessus:22964"]:
            templates = templates_api.get_templates_by_tag(tag)
            for template in templates:
                templates_api.delete_template(template.id)

        # Upload finding templates
        p = subprocess.Popen(
            ["reptor", "nessus", "--upload-finding-templates"],
            stderr=subprocess.PIPE,
        )
        _, output = p.communicate()
        assert p.returncode == 0
        output_lines = output.decode().splitlines()
        assert output_lines[-1].startswith("Successfully uploaded ")
        assert "Uploaded finding template \"global\"." in output_lines
        assert "Uploaded finding template \"22964\"." in output_lines

    @pytest.mark.parametrize(
        "input_file,expected_include,expected_exclude",
        [("nessus_single_host", 1, 5), ("nessus_multi_host", 2, 10)],
    )
    def test_nessus_template_vars(self, input_file, expected_include, expected_exclude):
        input_path = pathlib.Path(os.path.dirname(__file__)) / f"data/{input_file}.xml"

        p = subprocess.Popen(
            ["reptor", "nessus", "--template-vars", "--include", "11219,25216"],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        output, _ = p.communicate(input=input_path.read_bytes())
        assert p.returncode == 0
        parsed = json.loads(output)
        assert len(parsed) == expected_include

        p = subprocess.Popen(
            ["reptor", "nessus", "--template-vars", "--exclude", "11219,25216"],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        output, _ = p.communicate(input=input_path.read_bytes())
        assert p.returncode == 0
        parsed = json.loads(output)
        assert len(parsed) == expected_exclude

    @pytest.mark.parametrize(
        "input_file,expected_len",
        [("nessus_single_host", 1), ("nessus_multi_host", 4)],
    )
    def test_nessus_parse(self, input_file, expected_len):
        input_path = pathlib.Path(os.path.dirname(__file__)) / f"data/{input_file}.xml"

        p = subprocess.Popen(
            ["reptor", "nessus", "--parse"],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        output, _ = p.communicate(input=input_path.read_bytes())
        assert p.returncode == 0
        parsed = json.loads(output)
        assert len(parsed) == expected_len

    def test_nessus_note(self, notes_api):
        input_path = (
            pathlib.Path(os.path.dirname(__file__)) / "data/nessus_single_host.xml"
        )

        p = subprocess.Popen(
            ["reptor", "nessus", "--upload"],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        p.communicate(input=input_path.read_bytes())
        assert p.returncode == 0

        note = notes_api.get_note_by_title(
            "🟢 Nessus SYN scanner", parent_notetitle="preprod.boardvantage.net"
        )
        note_lines = note.text.splitlines()
        lines = [
            "Port 443/tcp was found to be open",
            "**Severity:** Info  ",
        ]
        assert all([line in note_lines for line in lines])

    def test_nessus_findings(self, projects_api):
        nmap_output_path = (
            pathlib.Path(os.path.dirname(__file__)) / "data/nessus_single_host.xml"
        )

        p = subprocess.Popen(
            ["reptor", "nessus", "--push-findings"],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        p.communicate(input=nmap_output_path.read_bytes())
        assert p.returncode == 0

        projects_api.get_findings()
        assert "Nessus SYN scanner" in [
            f.data.title for f in projects_api.get_findings()
        ]

    def test_conf(self):
        # Must be last test as it modifies the config file
        # Test interactive config
        p = subprocess.Popen(
            ["reptor", "nessus", "--conf"],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            # stderr=subprocess.PIPE,
        )
        read_until(p.stdout)
        p.stdin.write(b"medium-critical\n")  # type: ignore
        p.stdin.flush()  # type: ignore

        read_until(p.stdout)
        p.stdin.write(b"1234,9876\n")  # type: ignore
        p.stdin.flush()  # type: ignore

        read_until(p.stdout)
        p.stdin.write(b"4567,5678\n")  # type: ignore
        p.stdin.flush()  # type: ignore

        read_until(p.stdout)
        p.stdin.write(b"y\n")  # type: ignore
        p.stdin.flush()  # type: ignore
        p.wait(timeout=5)

        # Assert config file was created correctly
        with open(pathlib.Path.home() / ".sysreptor/config.yaml") as f:
            config = yaml.safe_load(f)["nessus"]

        assert all(
            [s in ["medium", "high", "critical"] for s in config["severity_filter"]]
        )
        assert config["excluded_plugins"] == ["1234", "9876"]
        assert config["included_plugins"] == ["4567", "5678"]



import re
import typing
from copy import deepcopy

from reptor.lib.plugins.ToolBase import ToolBase
from reptor.models.Note import NoteTemplate
from reptor.models.UserConfig import UserConfig


class Burp(ToolBase):
    meta = {
        "author": "Syslifters",
        "name": "Burp",
        "version": "1.0",
        "license": "MIT",
        "tags": [],
        "summary": "Burp vulnerability importer",
    }

    risk_mapping = {
        "info": "🟢",
        "low": "🔵",
        "medium": "🟡",
        "high": "🟠",
    }
    group_fields = ["serialNumber", "host", "path", "location", "collaboratorEvent"]
    remove_fields = ["requestresponse"]
    supports_multi_input = True  # TODO

    def _parse_severity_filter(self, filter: str) -> set:
        if filter is None:
            return set(self.risk_mapping.keys())
        filter_elements = set()
        if "-" in filter:
            filter_elements = filter.split("-")
            try:
                filter_indexes = [
                    list(self.risk_mapping.keys()).index(f) for f in filter_elements
                ]
            except ValueError:
                raise ValueError(
                    "Invalid filter range. Use keywords from 'info,low,medium,high'"
                )
            filter_elements = list()
            for i in range(min(filter_indexes), max(filter_indexes) + 1):
                filter_elements += [list(self.risk_mapping.keys())[i]]
        elif filter:
            filter_elements = filter.split(",")
            if not all(f in self.risk_mapping for f in filter_elements):
                raise ValueError(
                    "Invalid filter. Use keywords from 'info,low,medium,high'"
                )
        if filter_elements:
            return set(filter_elements)
        else:
            return set(self.risk_mapping.keys())

    @property
    def user_config(self) -> typing.List[UserConfig]:
        return [
            UserConfig(
                name="severity_filter",
                friendly_name='Severity filter (e.g., "medium-high")',
                callback=self._parse_severity_filter,
            ),
            UserConfig(
                name="excluded_plugins",
                friendly_name="Exclude plugin IDs (comma-separated)",
                callback=UserConfig.split,
            ),
            UserConfig(
                name="included_plugins",
                friendly_name="Include plugin IDs (comma-separated)",
                callback=UserConfig.split,
            ),
        ]

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.notetitle = kwargs.get("notetitle") or "Burp"
        self.note_icon = "🟧"
        self.input_format = "xml"
        self.timestamp = False
        self.severity_filter = getattr(
            self, "severity_filter", None
        ) or self._parse_severity_filter(kwargs.get("severity_filter", "info-high"))
        self.included_plugins = getattr(self, "included_plugins", list())
        self.included_plugins += list(
            filter(None, ((kwargs.get("included_plugins")) or "").split(","))
        )
        self.excluded_plugins = getattr(self, "excluded_plugins", list())
        self.excluded_plugins += list(
            filter(None, ((kwargs.get("excluded_plugins")) or "").split(","))
        )

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath=plugin_filepath)
        parser.add_argument(
            "--severity-filter",
            help='Filter findings by severity comma-separated ("info,low,medium,high") or as range ("medium-high")',
            action="store",
            dest="severity_filter",
            default=None,
        )
        parser.add_argument(
            "--exclude",
            help="Exclude plugin IDs, comma-separated",
            action="store",
            dest="excluded_plugins",
            default=None,
        )
        parser.add_argument(
            "--include",
            help="Include plugin IDs, comma-separated; default: all are included",
            action="store",
            dest="included_plugins",
            default=None,
        )

    def create_notes(self):
        self.parse()
        # Create note structure
        ## Main note
        main_note = NoteTemplate()
        main_note.title = self.notetitle
        main_note.icon_emoji = self.note_icon

        ## Subnotes per target
        for ip, findings in self.aggregate_by_ip().items():
            host_overview = NoteTemplate()
            host_overview.title = ip
            host_overview.checked = False
            host_overview.template = "burp-host-overview"
            host_overview.template_data = {"findings": findings, "ip": ip}
            for finding in findings:
                finding_note = NoteTemplate()
                finding_note.title = f"{ self.risk_mapping.get(finding.get('severity', 'info').lower()) } {finding['name']}"
                finding_note.checked = False
                finding_note.template = "burp-finding"
                finding_note.template_data = finding
                finding_note.force_new = True
                host_overview.children.append(finding_note)
            main_note.children.append(host_overview)
        if not main_note.children:
            return None
        return main_note

    def parse(self):
        super().parse()
        if not self.parsed_input:
            self.display("No data to parse.")
            return
        if isinstance(self.parsed_input, dict):
            self.parsed_input = [self.parsed_input]
        elif not isinstance(self.parsed_input, list):
            self.warning("Cannot handle this format. Expected dict input.")
            return

        findings = [finding for input in self.parsed_input for finding in input.get("issues", {}).get("issue", [])]

        # Exclude False Positives
        findings = [f for f in findings if f.get("severity") != "False Positive"]

        # Normalize severity
        for finding in findings:
            finding["severity"] = (finding.get("severity") or "info").lower()
            if finding["severity"] == "information":
                finding["severity"] = "info"

        # Apply filters
        if self.included_plugins:
            findings = [f for f in findings if f.get("type") in self.included_plugins]
        if self.excluded_plugins:
            findings = [
                f for f in findings if f.get("type") not in self.excluded_plugins
            ]
        if self.severity_filter:
            if any(f not in self.risk_mapping for f in self.severity_filter):
                raise ValueError(
                    f"Invalid severity filter. Use keywords from {', '.join(self.risk_mapping.keys())}"
                )
            findings = [
                f
                for f in findings
                if (f.get("severity") or "").lower() in self.severity_filter
            ]

        # References to list
        for finding in findings:
            finding["references"] = re.findall(
                r"<a\s+href=['\"](.*?)['\"]", finding.get("references") or ""
            )

        # Set severity score
        for finding in findings:
            finding["severity_score"] = list(self.risk_mapping.keys()).index(
                finding["severity"]
            )

        # Order by severity score
        self.parsed_input = sorted(
            findings, key=lambda x: x.get("severity_score", 0), reverse=True
        )

    def _remove_fields(self, finding: dict) -> dict:
        for field in self.remove_fields:
            finding.pop(field, None)
        return finding

    def _group_findings(self, findings: list) -> typing.Optional[dict]:
        if not findings:
            return None
        for field in self.group_fields:
            values = list()
            for finding in findings:
                if value := finding.get(field):
                    values.append(value)
            if values:
                findings[0][field] = values
        self._remove_fields(findings[0])
        return findings[0]

    def aggregate_by_ip(self) -> dict:
        finding_groups = dict()
        for finding in deepcopy(self.parsed_input) or list():
            if host := finding.get("host"):
                ip = host.get("@ip") or host.get("#text") or "_"
            else:
                ip = "_"
            finding_groups.setdefault(ip, list()).append(finding)

        for ip, findings in finding_groups.items():
            if ip != "_":
                finding_groups[ip] = self.merge_findings_by_type(findings)
        return finding_groups

    def merge_findings_by_type(self, findings) -> list:
        result = list()
        finding_groups = dict()
        for finding in deepcopy(findings) or list():
            if not (type := finding.get("type")):
                result.append(finding)
                continue
            finding_groups.setdefault(type, list()).append(finding)

        for type, findings in finding_groups.items():
            if not findings:
                continue
            # Create affected_components
            affected_components = list()
            for finding in findings:
                if not (host := finding.get("host")):
                    continue
                url = host.get("#text") or host.get("@ip")
                ip = host.get("@ip")
                location = finding.get("location") or finding.get("path") or ""
                affected_components.append(
                    f"{url or ip}{location}{f' ({ip})' if ip and url else ''}"
                )
            findings[0]["affected_components"] = affected_components

            # Add finding template information
            findings[0]["finding_templates"] = type

            # Group and append to aggregated findings
            result.append(self._group_findings(findings))

        # Order by severity score
        return sorted(result, key=lambda x: x.get("severity_score", 0), reverse=True)

    def preprocess_for_template(self) -> list:
        return self.merge_findings_by_type(self.parsed_input)

    def finding_global(self):
        return self.preprocess_for_template()


loader = Burp



import json
import subprocess

import pytest


@pytest.mark.integration
class TestIntegrationDeleteFinding(object):
    def test_delete_delete_projects(self):
        p = subprocess.Popen(
            ["reptor", "project", "--list", "--json"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        projects, _ = p.communicate()
        project_titles = [p["name"] for p in json.loads(projects)]

        # Delete dry run without search
        p = subprocess.Popen(
            ["reptor", "deleteprojects"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        _, lines = p.communicate()
        assert p.returncode == 0
        deleted_projects = [
            line.split(" ", 3)[-1].strip('"')
            for line in lines.decode().splitlines()
            if line.startswith("Would delete")
        ]
        assert all(p in project_titles for p in deleted_projects)

        # Delete dry run with search
        p = subprocess.Popen(
            ["reptor", "deleteprojects", "--title-contains", project_titles[0]],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        _, lines = p.communicate()
        assert p.returncode == 0
        deleted_projects = [
            line.split(" ", 3)[-1].strip('"')
            for line in lines.decode().splitlines()
            if line.startswith("Would delete")
        ]
        assert all(project_titles[0].lower() in p.lower() for p in deleted_projects)

        # Delete dry run with exclude
        p = subprocess.Popen(
            ["reptor", "deleteprojects", "--exclude-title-contains", project_titles[0]],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        _, lines = p.communicate()
        assert p.returncode == 0
        deleted_projects = [
            line.split(" ", 3)[-1].strip('"')
            for line in lines.decode().splitlines()
            if line.startswith("Would delete")
        ]
        assert all(project_titles[0].lower() not in p.lower() for p in deleted_projects)



import argparse
import io
import json
import tarfile
import uuid
from pathlib import Path

import tomli

from reptor.lib.plugins.Base import Base


def dir_path(path):
    p = Path(path)
    if not p.is_dir():
        raise argparse.ArgumentTypeError(f"readable_dir:{path} is not a valid path")
    return p


def build_tarinfo(name, size):
    info = tarfile.TarInfo(name=name)
    info.size = size
    return info


class PackArchive(Base):
    meta = {
        "name": "PackArchive",
        "summary": "Pack directories into a .tar.gz file",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.directories: list[Path] = kwargs.get("directories") or []
        self.output = kwargs.get("output")

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath=plugin_filepath)

        parser.add_argument("directories", nargs="+", type=dir_path)
        parser.add_argument(
            "-o",
            "--output",
            type=argparse.FileType("wb"),
            default="packed_archive.tar.gz",
        )

    def run(self):
        if not self.directories:
            return
        with tarfile.open(fileobj=self.output, mode="w:gz") as tar:
            for path_dir in self.directories:
                # Add NOTICE files at top level
                notice_path = path_dir / "NOTICE"
                if notice_path.is_file():
                    notice_filename = "NOTICE"
                    if notice_filename in tar.getnames():
                        i = 0
                        while True:
                            i += 1
                            notice_filename = f"NOTICE ({str(i)})"
                            if notice_filename not in tar.getnames():
                                break
                    tar.add(notice_path, arcname=notice_filename)

                # Add files to archive
                for path_input in list(path_dir.glob("*.toml")) + list(
                    path_dir.glob("*.json")
                ):
                    if not path_input.is_file():
                        continue

                    if path_input.suffix == ".toml":
                        data_dict = tomli.loads(path_input.read_text())
                    else:
                        data_dict = json.loads(path_input.read_text())

                    if not data_dict.get("id"):
                        data_dict["id"] = str(uuid.uuid4())
                    data_json = json.dumps(data_dict, indent=2)
                    tar.addfile(
                        build_tarinfo(
                            name=data_dict["id"] + ".json", size=len(data_json)
                        ),
                        fileobj=io.BytesIO(data_json.encode()),
                    )

                    # Add files to archive
                    # Translate human-friendly names to archive names based on IDs
                    file_dirs = {}
                    if data_dict.get("format").startswith("projects/"):
                        project_type_id = data_dict.get("project_type", {}).get("id")
                        file_dirs |= {
                            f"{data_dict['id']}-images": f"{data_dict['id']}-images",
                            f"{data_dict['id']}-files": f"{data_dict['id']}-files",
                            f"{project_type_id}-assets": f"{project_type_id}-assets",
                            f"{path_input.stem}-images": f"{data_dict['id']}-images",
                            f"{path_input.stem}-files": f"{data_dict['id']}-files",
                            f"{path_input.stem}-assets": f"{project_type_id}-assets",
                        }
                    elif data_dict.get("format").startswith("projecttypes/"):
                        file_dirs |= {
                            f"{data_dict['id']}-assets": f"{data_dict['id']}-assets",
                            f"{path_input.stem}-assets": f"{data_dict['id']}-assets",
                        }
                    elif data_dict.get("format").startswith("templates/"):
                        file_dirs |= {
                            f"{data_dict['id']}-images": f"{data_dict['id']}-images",
                            f"{path_input.stem}-images": f"{data_dict['id']}-images",
                        }

                    for ds, dd in file_dirs.items():
                        d_dir = Path(path_dir) / ds
                        if d_dir.is_dir():
                            tar.add(d_dir, arcname=dd)

        self.success(f"Packed contents to {self.output.name}")


loader = PackArchive



import copy
import os
from unittest.mock import MagicMock

import pytest

from reptor.lib.reptor import Reptor

from ..ProjectsAPI import ProjectsAPI


class TestConf(object):
    def test_config_from_environment(self):
        reptor_file = Reptor()
        reptor_file_config = copy.copy(reptor_file._config._raw_config)
        os.environ["REPTOR_SERVER"] = "https://demo1234.sysre.pt"
        os.environ["REPTOR_TOKEN"] = "sysreptor_abcdef"
        os.environ["REPTOR_PROJECT_ID"] = "2b5de38d-2932-4112-b0f7-42c4889dd64d"
        reptor_environ = Reptor()

        assert (
            reptor_file_config["server"] != reptor_environ._config._raw_config["server"]
        )
        assert (
            reptor_environ._config._raw_config["server"] == "https://demo1234.sysre.pt"
        )
        assert (
            reptor_file_config["token"] != reptor_environ._config._raw_config["token"]
        )
        assert reptor_environ._config._raw_config["token"] == "sysreptor_abcdef"
        assert (
            reptor_file_config["project_id"]
            != reptor_environ._config._raw_config["project_id"]
        )
        assert (
            reptor_environ._config._raw_config["project_id"]
            == "2b5de38d-2932-4112-b0f7-42c4889dd64d"
        )

        del os.environ["REPTOR_SERVER"]
        del os.environ["REPTOR_TOKEN"]
        del os.environ["REPTOR_PROJECT_ID"]


class TestProjectsAPI:
    class MockedCheckResponse:
        check_response = {
            "messages": [
                {
                    "level": "warning",
                    "message": "Unresolved TODO",
                    "details": "TODO: Executive Summary**",
                    "location": {
                        "type": "section",
                        "id": "executive_summary",
                        "name": "Executive Summary",
                        "path": "executive_summary",
                    },
                },
                {
                    "level": "warning",
                    "message": "Unresolved TODO",
                    "details": "TODO company",
                    "location": {
                        "type": "section",
                        "id": "customer",
                        "name": "Customer",
                        "path": "customer_name",
                    },
                },
            ]
        }

        def json(self):
            return self.check_response

    @pytest.fixture(autouse=True)
    def setUp(self):
        self.reptor = Reptor()
        self.reptor._config._raw_config["server"] = "https://demo.sysre.pt"
        self.reptor._config._raw_config["project_id"] = (
            "2b5de38d-2932-4112-b0f7-42c4889dd64d"
        )
        self.project = ProjectsAPI(reptor=self.reptor)

    def test_render(self):
        class MockGenerateResponse:
            content = b"binary-data"

        self.project.check_report = MagicMock(
            return_value={
                "Unresolved TODO": self.MockedCheckResponse.check_response["messages"]
            }
        )
        self.project.post = MagicMock(return_value=MockGenerateResponse())
        self.project.log.warning = MagicMock()

        self.project.render()
        self.project.check_report.assert_called_once()
        self.project.post.assert_called_once()
        self.project.log.warning.assert_called_once_with(
            'Report Check Warning: "Unresolved TODO" (x2)'
        )

    def test_check_report(self):
        self.project.get = MagicMock(return_value=self.MockedCheckResponse())
        assert self.project.check_report() == self.MockedCheckResponse.check_response
        assert self.project.check_report(group_messages=True) == {
            "Unresolved TODO": self.MockedCheckResponse.check_response["messages"]
        }

    def test_project_api_init(self):
        # Test valid init
        self.reptor._config._raw_config["server"] = "https://demo.sysre.pt"
        self.reptor._config._raw_config["project_id"] = (
            "2b5de38d-2932-4112-b0f7-42c4889dd64d"
        )
        try:
            ProjectsAPI(reptor=self.reptor).object_endpoint
        except ValueError:
            self.fail("ProjectsAPI raised ValueError")

        # Test missing server
        self.reptor._config._raw_config["server"] = ""
        with pytest.raises(ValueError):
            ProjectsAPI(reptor=self.reptor)

        # Test missing project id
        self.reptor._config._raw_config["server"] = "https://demo.sysre.pt"
        self.reptor._config._raw_config["project_id"] = ""
        with pytest.raises(ValueError):
            ProjectsAPI(reptor=self.reptor).object_endpoint



import argparse

from reptor.lib.plugins.Base import Base


class CreateProject(Base):
    """
    Create a new pentest project
    """

    meta = {
        "name": "CreateProject",
        "summary": "Create a new pentest project",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.name: str = kwargs.get("name") or "New Project created with Reptor"
        self.design: str = kwargs.get("design") or ""
        self.tags: list = ["reptor"]
        tags = kwargs.get("tags")
        if tags:
            self.tags.extend(tags.split(","))
        self.no_update_config: bool = kwargs.get("no_update_config", False)

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath=plugin_filepath)
        parser.add_argument(
            "-n",
            "--name",
            metavar="PROJECT NAME",
            help="Project name",
            action="store",
            default=None,
        )
        parser.add_argument(
            "-d",
            "--design",
            metavar="DESIGN ID",
            help="Design UUID for the project",
            action="store",
            required=True,
        )
        parser.add_argument(
            "-t",
            "--tags",
            metavar="TAGS",
            help="Comma-separated project tags",
            action="store",
            default=None,
        )
        parser.add_argument(
            "--no-update-config",
            action="store_true",
            help="Do not update project ID in config file",
            dest="no_update_config",
        )
        parser.add_argument(
            "--noupdateconfig",
            "--noupdate-config",
            "--no-updateconfig",
            action="store_true",
            help=argparse.SUPPRESS,
            dest="no_update_config",
        )

    def run(self):
        project = self.reptor.api.projects.create_project(
            name=self.name,
            project_design=self.design,
            tags=self.tags,
        )
        self.success(
            f'Successfully created "{self.name}" project with ID "{project.id}".'
        )

        if self.no_update_config:
            return

        self.reptor.api.projects.switch_project(project.id)
        config_from_file = self.reptor._config.load_config(return_only=True)
        config_from_file["project_id"] = project.id
        if config_from_file and not self.reptor._config._no_config_file:
            self.reptor._config._write_to_file(config=config_from_file)
        self.display("Updated project ID in config file.")
        return project


loader = CreateProject



import itertools
import typing
from typing import Any

from reptor.models.Base import BaseModel, ProjectFieldTypes
from reptor.settings import DEFAULT_PROJECT_DESIGN


class ProjectDesignField(BaseModel):
    """

    Attributes:
        name:
        type:
        label:
        origin:
        default:
        required:
        spellcheck:

        properties: "ProjectDesignField"
        choices:
        items:
        suggestions:
    """

    id: str = ""
    type: ProjectFieldTypes
    label: str = ""
    origin: str = ""
    default: str = ""
    required: bool = False
    spellcheck: bool = False
    # Use Any instead of "typing.List['ProjectDesignField'] = []" due to Python bug
    # See: https://bugs.python.org/issue44926
    properties: Any = None
    choices: typing.List[dict] = []
    items: dict = {}
    suggestions: typing.List[str] = []

    @property
    def name(self):
        return self.id

    def _fill_from_api(self, data: typing.Dict):
        data = data.copy()
        if 'id' not in data and 'name' in data:
            data['id'] = data['name']

        if data["type"] == ProjectFieldTypes.list.value:
            if isinstance(data["items"], dict):
                data["items"] = ProjectDesignField(data["items"])
        elif data["type"] == ProjectFieldTypes.object.value:
            if isinstance(data["properties"], dict):
                data["properties"] = [ProjectDesignField(f | {'id': fid}) for fid, f in data["properties"].items()]
            elif isinstance(data['properties'], list):
                data["properties"] = [ProjectDesignField(f) for f in data["properties"]]

        attrs = typing.get_type_hints(self).keys()
        for key, value in data.items():
            if key in attrs:
                self.__setattr__(key, value)


class ProjectDesignBase(BaseModel):
    source: str = ""
    scope: str = ""
    name: str = ""
    language: str = ""

    def __init__(self, data: typing.Optional[typing.Dict] = None):
        if data is None:
            data = DEFAULT_PROJECT_DESIGN
        super().__init__(data)


class ProjectDesign(ProjectDesignBase):
    report_fields: typing.List[ProjectDesignField] = []
    finding_fields: typing.List[ProjectDesignField] = []

    def __init__(self, data: typing.Optional[typing.Dict] = None):
        if data:
            if isinstance(data.get("report_fields"), str):
                raise ValueError(
                    "report_fields should be list. Use ProjectDesignOverview instead."
                )
            if isinstance(data.get("finding_fields"), str):
                raise ValueError(
                    "finding_fields should be list. Use ProjectDesignOverview instead."
                )
        super().__init__(data)

    def _fill_from_api(self, data: typing.Dict):
        report_fields = []
        if isinstance(data.get('report_fields'), dict):
            report_fields = [ProjectDesignField(f | {'id': fid}) for fid, f in data['report_fields'].items()]
        else:
            for field in itertools.chain(*map(lambda s: s['fields'], data.get('report_sections', []))):
                if isinstance(field, dict):
                    report_fields.append(ProjectDesignField(field))
        
        finding_fields = []
        if isinstance(data.get('finding_fields'), dict):
            finding_fields = [ProjectDesignField(f | {'id': fid}) for fid, f in data['finding_fields'].items()]
        elif isinstance(data.get('finding_fields'), list):
            finding_fields = [ProjectDesignField(f) for f in data['finding_fields']]

        super()._fill_from_api(data | {'report_fields': report_fields, 'finding_fields': finding_fields})


class ProjectDesignOverview(ProjectDesignBase):
    report_fields: str = ""
    finding_fields: str = ""

    def __init__(self, data: typing.Optional[typing.Dict] = None):
        if data:
            if isinstance(data.get("report_fields"), list):
                raise ValueError(
                    "report_fields should be str. Use ProjectDesign instead."
                )
            if isinstance(data.get("finding_fields"), list):
                raise ValueError(
                    "finding_fields should be str. Use ProjectDesign instead."
                )
        super().__init__(data)



import json
import os
import shutil
import subprocess
from pathlib import Path

import pytest
import tomli


@pytest.mark.integration
class TestUnPackArchive(object):
    input_path = Path(os.path.dirname(__file__)) / "data/unpacked"
    archive_path = Path(os.path.dirname(__file__)) / "data/packed_archive.tar.gz"
    unpacked_path = Path(os.path.dirname(__file__)) / "data/unpacked_archive"

    @pytest.fixture(autouse=True)
    def tearDown(self):
        yield
        # Delete output files, ignore errors
        for path in [self.archive_path, self.unpacked_path]:
            try:
                if os.path.isfile(path):
                    path.unlink()
                else:
                    shutil.rmtree(path)
            except FileNotFoundError:
                pass

    @pytest.mark.parametrize("format", ["toml"])
    def test_pack_unpack(self, format):
        # Pack archive
        p = subprocess.Popen(
            [
                "reptor",
                "packarchive",
                "--output",
                str(self.archive_path),
                str(self.input_path),
            ],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        p.communicate()
        assert p.returncode == 0

        # Unpack archive
        p = subprocess.Popen(
            [
                "reptor",
                "unpackarchive",
                "--format",
                format,
                "--output",
                str(self.unpacked_path),
                str(self.archive_path),
            ],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        p.communicate()
        assert p.returncode == 0

        # Compare unpacked files
        assert set(os.listdir(self.input_path)) == set(os.listdir(self.unpacked_path))
        original = tomli.loads((self.input_path / "project.toml").read_text())
        if format == "json":
            unpacked = json.loads((self.unpacked_path / "project.json").read_text())
        elif format == "toml":
            unpacked = tomli.loads((self.unpacked_path / "project.toml").read_text())
        else:
            raise ValueError(f"Unknown format: {format}")
        assert unpacked == original



import argparse
import re
from typing import Union

from requests.exceptions import HTTPError

from reptor.lib.plugins.Base import Base
from reptor.models.Base import ProjectFieldTypes
from reptor.models.Finding import Finding
from reptor.models.Section import Section
from reptor.models.UserConfig import UserConfig

try:
    import deepl
except ImportError:
    deepl = None


class Translate(Base):
    """ """

    meta = {
        "name": "Translate",
        "summary": "Translate Projects to other languages via Deepl",
    }

    PREDEFINED_SKIP_FIELDS = [
        "affected_components",
        "references",
    ]
    TRANSLATE_DATA_TYPES = [
        ProjectFieldTypes.string.value,
        ProjectFieldTypes.markdown.value,
    ]
    DEEPL_FROM = [
        "BG",
        "CS",
        "DA",
        "DE",
        "EL",
        "EN",
        "ES",
        "ET",
        "FI",
        "FR",
        "HU",
        "ID",
        "IT",
        "JA",
        "KO",
        "LT",
        "LV",
        "NB",
        "NL",
        "PL",
        "PT",
        "RO",
        "RU",
        "SK",
        "SL",
        "SV",
        "TR",
        "UK",
        "ZH",
    ]
    DEEPL_TO = [
        "BG",
        "CS",
        "DA",
        "DE",
        "EL",
        "EN",
        "EN-GB",
        "EN-US",
        "ES",
        "ET",
        "FI",
        "FR",
        "HU",
        "ID",
        "IT",
        "JA",
        "KO",
        "LT",
        "LV",
        "NB",
        "NL",
        "PL",
        "PT",
        "PT-BR",
        "PT-PT",
        "RO",
        "RU",
        "SK",
        "SL",
        "SV",
        "TR",
        "UK",
        "ZH",
    ]

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.from_lang = kwargs.get("from")
        self.to_lang = kwargs["to"]
        self.dry_run = kwargs.get("dry_run")
        self.chars_count_to_translate = 0
        self.skip_fields = kwargs.get("skip_fields", None) or getattr(
            self, "skip_fields", None
        )
        if self.skip_fields:
            self.skip_fields = self.skip_fields.split(",")
        else:
            self.skip_fields = list()
        try:
            self.skip_fields.extend(self.PREDEFINED_SKIP_FIELDS)
        except TypeError:
            raise TypeError(f"skip_fields should be list.")
        if not hasattr(self, "deepl_api_token"):
            self.deepl_api_token = ""

        try:
            if not self.deepl_api_token:
                raise AttributeError("No Deepl API token found. Try --conf.")
            if not deepl:
                raise ModuleNotFoundError(
                    'deepl library not found. Install plugin requirements with "pip3 install reptor[translate]'
                )
            self.deepl_translator = deepl.Translator(self.deepl_api_token)
        except (AttributeError, ModuleNotFoundError) as e:
            if not self.dry_run:
                raise e

    @property
    def user_config(self):
        return [
            UserConfig(
                name="deepl_api_token",
                friendly_name="Deepl API Token",
                redact_current_value=True,
            )
        ]

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath=plugin_filepath)
        parser.add_argument(
            "--from",
            metavar="LANGUAGE_CODE",
            help="Language code of source language",
            choices=cls.DEEPL_FROM,
            action="store",
            default=None,
        )
        parser.add_argument(
            "--to",
            metavar="LANGUAGE_CODE",
            help="Language code of dest language",
            choices=cls.DEEPL_TO,
            action="store",
            default=None,
        )
        parser.add_argument(
            "--skip-fields",
            metavar="FIELDS",
            help="Report and Finding fields, comma-separated",
            action="store",
            default="",
        )
        parser.add_argument(
            "--skipfields",
            metavar="FIELDS",
            help=argparse.SUPPRESS,
            action="store",
            default="",
        )
        parser.add_argument(
            "--dry-run",
            dest="dry_run",
            help="Do not translate, count characters to be translated and checks Deepl quota",
            action="store_true",
        )
        parser.add_argument(
            "--dryrun",
            dest="dry_run",
            help=argparse.SUPPRESS,
            action="store_true",
        )

    def _translate(self, text: str) -> str:
        if not re.search("[a-zA-Z]", text):
            return text

        self.chars_count_to_translate += len(text)

        result = self.deepl_translator.translate_text(
            text,
            source_lang=self.from_lang,  # Can be None
            target_lang=self.to_lang,
            preserve_formatting=True,
        )
        return result.text  # type: ignore

    def _translate_section(
        self, section: Union[Finding, Section]
    ) -> Union[Finding, Section]:
        for field in section.data:
            if field.type not in self.TRANSLATE_DATA_TYPES:
                continue
            if field.name in self.skip_fields:
                continue
            if not field.value:
                continue
            field.value = self._translate(field.value)
        return section

    def _dry_run_translate(self, text: str) -> str:
        self.chars_count_to_translate += len(text)
        return text

    def _duplicate_and_update_project(self, project_title: str) -> None:
        self.display(f"Duplicating project{' (dry run)' if self.dry_run else ''}.")
        if not self.dry_run:
            to_project_id = self.reptor.api.projects.duplicate_project().id
            self.display(
                f"Updating project metadata{' (dry run)' if self.dry_run else ''}."
            )
            # Switch project to update duplicated project instead of original
            self.reptor.api.projects.switch_project(to_project_id)

            try:
                data = {"name": self._translate(project_title)}
                if sysreptor_language_code := self._get_sysreptor_language_code(
                    self.to_lang
                ):
                    data["language"] = sysreptor_language_code
                self.reptor.api.projects.update_project(data)
            except HTTPError as e:
                self.warning(f"Error updating project: {e.response.text}")
        else:
            self._translate(project_title)  # To count characters

    def _translate_project(self):
        if self.dry_run:
            self._translate = self._dry_run_translate

        project = self.reptor.api.projects.project
        self._duplicate_and_update_project(project_title=project.name)

        self.display(f"Translating findings{' (dry run)' if self.dry_run else ''}.")
        sections = [
            Finding(f, self.reptor.api.project_designs.project_design)
            for f in self.reptor.api.projects.get_findings()
        ] + self.reptor.api.projects.get_sections()
        for section in sections:
            translated_section = self._translate_section(section)
            translated_section_data = translated_section.data.to_dict()
            if not self.dry_run:
                if translated_section.__class__ == Finding:
                    self.reptor.api.projects.update_finding(
                        translated_section.id, {"data": translated_section_data}
                    )
                elif translated_section.__class__ == Section:
                    self.reptor.api.projects._update_section(
                        translated_section.id, {"data": translated_section_data}
                    )

        self.display(
            f"Translated {self.chars_count_to_translate} characters{' (dry run)' if self.dry_run else ''}."
        )
        self._log_deepl_usage()
        self.success(f"Project translated{' (dry run)' if self.dry_run else ''}.")
        self.display(
            "We recommend to check quality of the translation, or to add a note that the report was "
            "translated by a machine."
        )

    def _log_deepl_usage(self):
        try:
            usage = self.deepl_translator.get_usage()
            if usage.any_limit_reached:
                self.warning("Deepl transaction limit reached.")
            if usage.character.valid:
                self.display(
                    f"Deepl usage: {usage.character.count} of {usage.character.limit} characters"
                )
        except AttributeError:
            pass

    def _get_sysreptor_language_code(self, language_code) -> str:
        enabled_language_codes = self.reptor.api.projects.get_enabled_language_codes()
        matched_lcs = [
            enabled_lc
            for enabled_lc in enabled_language_codes
            if enabled_lc.lower().startswith(language_code[:2].lower())
        ]
        if not matched_lcs:
            return ""
        elif len(matched_lcs) == 1:
            return matched_lcs[0]
        else:
            for matched_lc in matched_lcs:
                if matched_lc.lower() == language_code.lower():
                    return matched_lc
            else:
                return matched_lcs[0]

    def run(self):
        self._translate_project()


loader = Translate



import typing
from uuid import UUID

from requests.exceptions import HTTPError

from reptor.lib.plugins.UploadBase import UploadBase
from reptor.models.FindingTemplate import FindingTemplate


class FindingFromTemplate(UploadBase):
    meta = {
        "name": "FindingFromTemplate",
        "summary": "Creates findings from remote finding templates",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.template_id = kwargs.get("template_id", None)
        self.tags = (kwargs.get("tags") or "").split(",")

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath=plugin_filepath)
        action_group = parser.add_argument_group()
        action_group.add_argument(
            "--template-id",
            metavar="TEMPLATE_ID",
            action="store",
            nargs="?",
            help="UUID of the template to use",
        )
        action_group.add_argument(
            "--tags",
            metavar="TAGS",
            action="store",
            help="Create findings from finding templates with the specified tags; comma-separated list of tags",
        )


    def _get_template_by_id(self, template_id) -> FindingTemplate:
        try:
            UUID(template_id)
        except ValueError:
            raise ValueError(f'template_id "{template_id}" is not a UUID')
        try:
            template = self.reptor.api.templates.get_template(template_id)
        except HTTPError as e:
            if e.response.status_code == 404:
                raise KeyError(f"Template with ID {template_id} doesn't exist")
            else:
                raise HTTPError(f"Failed to get template: {e}") from e
        return template

    def _get_templates_by_tags(self, tags) -> typing.List[FindingTemplate]:
        templates = [
            t
            for tag in tags
            for t in self.reptor.api.templates.get_templates_by_tag(tag)
        ]
        upload_templates = list()
        upload_template_ids = set()
        for template in templates:
            if template.id in upload_template_ids:
                continue
            if all(tag in template.tags for tag in tags):
                upload_templates.append(template)
                upload_template_ids.add(template.id)
        if not upload_templates:
            raise KeyError(f"No templates found with tags: {', '.join(tags)}")
        return upload_templates


    def _get_template_translation(self, finding_template: FindingTemplate) -> typing.Tuple[int, str]:
        for i, t in enumerate(finding_template.translations):
            if t.language == self.reptor.api.projects.project.language:
                break
        else:
            for i, t in enumerate(finding_template.translations):
                if t.is_main:
                    return i, t.language
        return i, t.language


    def run(self):
        if self.template_id:
            upload_templates = [self._get_template_by_id(self.template_id)]
        elif self.tags:
            upload_templates = self._get_templates_by_tags(self.tags)
        else:
            raise ValueError("No template_id or tags provided (use --template-id or --tags)")

        for template in upload_templates:
            translation_index, language = self._get_template_translation(template)
            template_title = template.translations[translation_index].data.title
            
            if template.id in [f.template for f in self.reptor.api.projects.project.findings]:
                self.display(
                    f'Skipping template "{template_title}" as it already exists in the project'
                )
                continue
            
            try:
                self.reptor.api.projects.create_finding_from_template(template.id, language=language)
            except HTTPError as e:
                if template_msgs := e.response.json().get("template", list()):
                    template_msg = '\n'.join(template_msgs)
                    self.log.error(
                        f"{template_msg}"
                    )
                else:
                    raise Exception(f'Failed to create finding from template "{template_title}": {e}')

            self.success(
                f'Successfully created finding "{template_title}"'
            )

loader = FindingFromTemplate



from reptor.lib.plugins.PluginMeta import PluginMeta


class TestPluginMeta:
    example_docs = {
        "author": "Willy Wonka",
        "version": "1.5",
        "website": "https://github.com/Syslifters/reptor",
        "license": "MIT",
        "summary": "Parses OWASPZap XML and JSON reports",
    }

    def test_perfect_docs(self):
        plugin_meta = PluginMeta(self.example_docs)

        assert plugin_meta.author == "Willy Wonka"
        assert plugin_meta.version == "1.5"
        assert plugin_meta.website == "https://github.com/Syslifters/reptor"
        assert plugin_meta.license == "MIT"
        assert plugin_meta.summary == "Parses OWASPZap XML and JSON reports"



import datetime
import enum
import logging
import typing
from copy import deepcopy
from dataclasses import dataclass


@dataclass
class BaseModel:
    """
    Base Model
    """

    id: str = ""
    created: datetime.datetime = datetime.datetime.now()
    updated: datetime.datetime = datetime.datetime.now()
    _log = logging.getLogger("reptor")

    def __init__(self, data: typing.Optional[typing.Dict] = None):
        if data:
            self._fill_from_api(data)

    def _get_combined_class_type_hints(self) -> dict:
        type_hints = list()
        type_hints_from_class = self.__class__
        while type_hints_from_class.__base__:
            type_hints.append(typing.get_type_hints(type_hints_from_class))
            if type_hints_from_class == BaseModel:
                break
            type_hints_from_class = type_hints_from_class.__base__

        combined_class_type_hints = dict()
        for type_hint in reversed(type_hints):
            combined_class_type_hints.update(type_hint)
        return combined_class_type_hints

    def _fill_from_api(self, data: typing.Dict):
        """Fills Model from reptor.api return JSON data

        Args:
            data (str): API Return Data
        """
        # Import here to prevent circular imports
        from reptor.models.Finding import FindingDataRaw
        from reptor.models.FindingTemplate import (FindingTemplate,
                                                   FindingTemplateTranslation)
        from reptor.models.Note import Note
        from reptor.models.Project import Project
        from reptor.models.ProjectDesign import (ProjectDesign,
                                                 ProjectDesignField)
        from reptor.models.Section import SectionDataRaw
        from reptor.models.User import User

        combined_class_type_hints = self._get_combined_class_type_hints()

        for attr in combined_class_type_hints.items():
            if attr[0] in data:
                # Check if one of our models, then proceed recursively
                try:
                    model_class = attr[1].__args__[0]
                    is_list = True
                except AttributeError:
                    model_class = attr[1]
                    is_list = False

                if model_class in [
                    User,
                    FindingTemplate,
                    FindingDataRaw,
                    SectionDataRaw,
                    Note,
                    Project,
                    ProjectDesign,
                    FindingTemplateTranslation,
                ]:
                    if is_list:
                        item_list = list()
                        for item in data[attr[0]]:
                            item_list.append(model_class(item))
                        self.__setattr__(attr[0], item_list)
                    else:
                        self.__setattr__(attr[0], model_class(data[attr[0]]))
                # elif model_class == ProjectDesignField:
                #     self.__setattr__(attr[0], list())
                #     for k, v in data[attr[0]].items():
                #         v["name"] = k
                #         self.__getattribute__(attr[0]).append(model_class(v))
                else:
                    # Fill each attribute
                    self.__setattr__(attr[0], data[attr[0]])

    def to_dict(self):
        dict_values = deepcopy(vars(self))
        for k, v in dict_values.items():
            if isinstance(v, datetime.datetime):
                dict_values[k] = v.isoformat()
            elif isinstance(v, list):
                try:
                    dict_values[k] = [i.to_dict() for i in v]
                except AttributeError:
                    dict_values[k] = v
            else:
                try:
                    dict_values[k] = v.to_dict()  # type: ignore
                except AttributeError:
                    pass
        return dict_values


class ProjectFieldTypes(enum.Enum):
    cvss = "cvss"
    string = "string"
    markdown = "markdown"
    list = "list"
    object = "object"
    enum = "enum"
    user = "user"
    combobox = "combobox"
    date = "date"
    number = "number"
    boolean = "boolean"


class FindingTemplateSources(enum.Enum):
    CREATED = "created"
    IMPORTED = "imported"
    IMPORTED_DEPENDENCY = "imported_dependency"
    CUSTOMIZED = "customized"
    SNAPSHOT = "snapshot"



import json
import subprocess

import pytest
import tomli
import yaml

from reptor.plugins.core.Conf.tests.conftest import get_note, projects_api


@pytest.mark.integration
class TestIntegrationExportFinding(object):
    def test_export_findings(self, projects_api):
        p = subprocess.Popen(
            ["reptor", "exportfindings", "--format", "json"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        out, _ = p.communicate()
        assert p.returncode == 0
        json_export = json.loads(out)

        p = subprocess.Popen(
            ["reptor", "exportfindings", "--format", "toml"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        out, _ = p.communicate()
        assert p.returncode == 0
        toml_export = tomli.loads(out.decode()).get("findings")  # TOML wraps in a dict

        p = subprocess.Popen(
            ["reptor", "exportfindings", "--format", "yaml"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        out, _ = p.communicate()
        assert p.returncode == 0
        yaml_export = yaml.safe_load(out.decode())

        assert json_export == toml_export == yaml_export

        p = subprocess.Popen(
            ["reptor", "exportfindings", "--format", "json", "--upload"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        out, _ = p.communicate()
        assert p.returncode == 0

        note = get_note("Uploads", None)
        note_last_line = note["text"].splitlines()[-1]  # type: ignore
        assert f"[{projects_api.project.name}.json]" in note_last_line



import contextlib
import io
import json
import tarfile
import tempfile
import uuid
from pathlib import Path

import pytest
import tomli_w

from reptor.plugins.utils.packarchive.packarchive import PackArchive


@contextlib.contextmanager
def mock_files(files, format="toml"):
    with tempfile.TemporaryDirectory() as tmpdir:
        tmpdir = Path(tmpdir)
        for filename, content in files.items():
            filename = tmpdir / Path(filename)
            if isinstance(content, dict) and not filename.suffix:
                filename = filename.with_suffix(f".{format}")
            filename.parent.mkdir(parents=True, exist_ok=True)

            if isinstance(content, bytes):
                filename.write_bytes(content)
            elif isinstance(content, str):
                filename.write_text(content)
            elif isinstance(content, dict) and format == "json":
                filename.write_text(json.dumps(content))
            elif isinstance(content, dict) and format == "toml":
                filename.write_text(tomli_w.dumps(content))
            else:
                raise Exception(f"Unhandled type: {type(content)}")

        yield tmpdir


def create_png_file() -> bytes:
    # 1x1 pixel PNG file
    # Source: https://commons.wikimedia.org/wiki/File:1x1.png
    return (
        b"\x89PNG\r\n\x1a\n\x00\x00\x00\r"
        + b"IHDR\x00\x00\x00\x01\x00\x00\x00\x01\x01\x03\x00\x00\x00%\xdbV\xca\x00\x00\x00\x03"
        + b"PLTE\x00\x00\x00\xa7z=\xda\x00\x00\x00\x01tRNS\x00@\xe6\xd8f\x00\x00\x00\n"
        + b"IDAT\x08\xd7c`\x00\x00\x00\x02\x00\x01\xe2!\xbc3\x00\x00\x00\x00IEND\xaeB`\x82"
    )


class TestPackExport:
    def pack(self, files, format):
        with mock_files(
            files=files, format=format
        ) as d, tempfile.TemporaryFile() as output:
            PackArchive(directories=[d], output=output).run()
            output.flush()
            output.seek(0)
            return tarfile.open(fileobj=io.BytesIO(output.read()), mode="r:gz")

    @pytest.mark.parametrize(
        ["format"],
        [
            ("toml",),
            ("json",),
        ],
    )
    def test_pack_project_name(self, format):
        project_id = str(uuid.uuid4())
        design_id = str(uuid.uuid4())
        tar = self.pack(
            files={
                f"project1.{format}": {
                    "id": project_id,
                    "format": "projects/v1",
                    "project_type": {
                        "id": design_id,
                    },
                },
                "project1-images/img1.png": create_png_file(),
                "project1-files/file1.txt": b"test",
                "project1-assets/file2.txt": b"test",
            },
            format=format,
        )
        assert set(tar.getnames()) == {
            f"{project_id}.json",
            f"{project_id}-images",
            f"{project_id}-images/img1.png",
            f"{project_id}-files",
            f"{project_id}-files/file1.txt",
            f"{design_id}-assets",
            f"{design_id}-assets/file2.txt",
        }

    @pytest.mark.parametrize(
        ["format"],
        [
            ("toml",),
            ("json",),
        ],
    )
    def test_pack_project_id(self, format):
        project_id = str(uuid.uuid4())
        design_id = str(uuid.uuid4())
        tar = self.pack(
            files={
                f"{project_id}.{format}": {
                    "id": project_id,
                    "format": "projects/v1",
                    "project_type": {
                        "id": design_id,
                        "format": "projecttype/v1",
                    },
                },
                f"{project_id}-images/img1.png": create_png_file(),
                f"{project_id}-files/file1.txt": b"test",
                f"{design_id}-assets/file2.txt": b"test",
            },
            format=format,
        )
        assert set(tar.getnames()) == {
            f"{project_id}.json",
            f"{project_id}-images",
            f"{project_id}-images/img1.png",
            f"{project_id}-files",
            f"{project_id}-files/file1.txt",
            f"{design_id}-assets",
            f"{design_id}-assets/file2.txt",
        }

    @pytest.mark.parametrize(
        ["format"],
        [
            ("toml",),
            ("json",),
        ],
    )
    def test_pack_template_name(self, format):
        template_id = str(uuid.uuid4())
        tar = self.pack(
            files={
                f"template1.{format}": {
                    "id": template_id,
                    "format": "templates/v1",
                },
                "template1-images/img1.png": create_png_file(),
            },
            format=format,
        )
        assert set(tar.getnames()) == {
            f"{template_id}.json",
            f"{template_id}-images",
            f"{template_id}-images/img1.png",
        }



import contextlib
import json
import sys
import typing

import tomli

from reptor.lib.plugins.UploadBase import UploadBase
from reptor.models.Finding import Finding as FindingModel
from reptor.models.ProjectDesign import ProjectDesign


class Finding(UploadBase):
    """ """

    meta = {
        "name": "Finding",
        "summary": "Uploads findings from JSON or TOML",
    }

    def run(self):
        findings = list(self._read_findings())
        for finding in findings:
            self.reptor.api.projects.create_finding(finding)
        findings_count = len(findings)
        self.log.success(
            f"Successfully uploaded {findings_count} finding{'s'[:findings_count^1]}"
        )

    def _read_findings(
        self, content: typing.Optional[str] = None
    ) -> typing.Iterator[dict]:
        if content is None:
            # Read finding from stdin
            self.display("Reading from stdin...")
            content = sys.stdin.read()

        loaded_content: typing.Union[None, dict, list] = None
        with contextlib.suppress(json.JSONDecodeError):
            loaded_content = json.loads(content, strict=False)
        if not loaded_content:
            with contextlib.suppress(tomli.TOMLDecodeError):
                loaded_content = tomli.loads(content)
        if not loaded_content:
            raise ValueError("Could not decode stdin (expected JSON or TOML)")

        if isinstance(loaded_content, dict):
            loaded_content = [loaded_content]

        for finding in loaded_content:
            # Create model to assure compatibility of predefined fields
            assert isinstance(finding, dict)
            FindingModel(finding, ProjectDesign(), raise_on_unknown_fields=False)
            yield finding


loader = Finding



import typing

from reptor.models.Base import BaseModel
from reptor.models.Finding import Finding
from reptor.models.ProjectDesign import ProjectDesign
from reptor.models.Section import Section
from reptor.models.User import User


class ProjectBase(BaseModel):
    name: str = ""
    project_type: str = ""  # is the project design id
    language: str = ""
    tags: typing.List[str] = []
    readonly: bool = False
    source: str = ""
    copy_of: str = ""
    override_finding_order: bool = False
    members: typing.List[User] = []
    imported_members: typing.List[User] = []
    details: str = ""
    notes: str = ""
    images: str = ""


class Project(ProjectBase):
    findings: typing.List[Finding] = []
    sections: typing.List[Section] = []

    def __init__(self, data: dict, project_design: ProjectDesign):
        if isinstance(data.get("findings"), str):
            raise ValueError("Findings should be list. Use ProjectOverview instead.")
        if isinstance(data.get("sections"), str):
            raise ValueError("Sections should be list. Use ProjectOverview instead.")
        super().__init__(data)
        self.findings = list()
        self.sections = list()
        for finding in data.get("findings", []):
            self.findings.append(Finding(finding, project_design))
        for section in data.get("sections", []):
            self.sections.append(Section(section, project_design))
        # Order findings by their order attribute
        if self.override_finding_order:
            self.findings.sort(key=lambda f: f.order)


class ProjectOverview(ProjectBase):
    findings: str = ""
    sections: str = ""

    def __init__(self, data: dict):
        if isinstance(data.get("findings"), list):
            raise ValueError("Findings should be str. Use Project instead.")
        if isinstance(data.get("sections"), list):
            raise ValueError("Sections should be str. Use Project instead.")
        super().__init__(data)
